<?php

/**
 * FME Extensions
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the fmeextensions.com license that is
 * available through the world-wide-web at this URL:
 * https://www.fmeextensions.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    FME
 * @package     FME_CopyCategory
 * @copyright   Copyright (c) 2019 FME (http://fmeextensions.com/)
 * @license     https://fmeextensions.com/LICENSE.txt
 */

namespace FME\CopyCategory\Console;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Input\InputOption;

class Copy extends Command {

    public $entity_id;
    protected $categoryFactory;
    protected $_categoryHelper;
    protected $categoryRepository;
    protected $categoryList;

    public function __construct(
        \Magento\Catalog\Model\CategoryFactory $categoryFactory, 
        \Magento\Store\Model\StoreManagerInterface $storeManager, 
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $date) 
    {
        $this->storeManager = $storeManager;
        parent::__construct();
        $this->_date = $date;
        $this->categoryFactory = $categoryFactory;
    }

    protected function configure() {

        $options = [
            new InputOption(
                    'q', null, InputOption::VALUE_REQUIRED, 'category Id'
            )
        ];

        $this->setName('copy:category')
                ->setDescription('Create Duplicate of category using CLI')
                ->setDefinition($options);

        parent::configure();
    }

    protected function execute(InputInterface $input, OutputInterface $output) {
        if ($name = $input->getOption('q')) {
            $stdObject = json_decode($name);
        }
        $data = json_decode(json_encode($stdObject), True);
        $id = $data['entity_id'];
        $this->entity_id = $id;

        $copydata = $data['setting'];
        if (isset($copydata['search']))
            $copydata['search'] = $copydata['search']['replacedata']['replacedata'];

        $newCatgoryid = $this->copyParentCategory($id, $copydata,$output);
        if ($copydata['copy_sub']):
            $oldParentid = $id;
            $tree = array();
            $this->copySubCategory($oldParentid, $newCatgoryid, $tree);
            $copydata['data'] = $data['setting'];
            if (is_array($copydata['sub_categories'])):
                $selected = array_keys($tree);
                $fliped = array_flip($copydata['sub_categories']);
                $result = array_diff_key($tree, $fliped);
                $output->writeln("\nCopying Sub-Categories...Please Wait");
                foreach ($result as $key => $value) {
                    unset($tree[$key]);
                }
            endif;
            $copydata['tree'] = $tree;
             
            array_walk($tree, array($this, 'saveSubCategory'), $copydata);
        endif;
        $output->writeln("You Copied Category Successfully.");
        return \Magento\Framework\Console\Cli::RETURN_SUCCESS;
    }

    public function copyParentCategory($id, $data,$output) {
        $parentdata = $this->getParentData($data['new_parent_category']);
        $model = $this->getCategory($id);
        if ($data['category_name'] !== "") {
            $model = $this->getCategory($id);
            $model->setName($data['category_name']);
            $key = str_replace(' ', '-', strtolower($data['category_name']));
            $model->setUrlKey($key);
        } else {
            $key = $model->getUrlKey();
        }
        if ($data['copy_related_products']):
            $positions = $model->getProductsPosition();
            $model->setPostedProducts($positions);
            $model->setCategoryProducts(json_encode($positions));
            unset($model['products_position']);
        endif;
        $model->setUrlPath($parentdata['urlkey'] . '/' . $key);
        $model->setParentId($parentdata['id']);
        $model->setEntityId(null);
        $model->setPath($parentdata['path']);
        $model->setPathIds($arrayName = array('0' => ''));
        $model->setCreatedAt($this->_date->date()->format('Y-m-d H:i:s'));
        $model->setUpdatedAt(null);
        try {
            $output->writeln("\nCopying Parent Category...");
            $model->save();
        } catch (\Exception $e) {
            $result = ['error' => $e->getMessage(), 'errorcode' => $e->getCode()];
        }
        return $model->getId();
    }

    public function copySubCategory($oldParentid, $newParentId, &$array_cat) {
        $categories = $this->getCategory($oldParentid);
        if ($categories->hasChildren()) {
            $subcategorid = explode(',', $categories->getChildren());
            foreach ($subcategorid as $categoryId) {
                $subcategory = $categories = $this->getCategory($categoryId);
                if ($subcategory->getParentId() === $this->entity_id):
                    $parentId = $newParentId;
                else:
                    $parentId = $subcategory->getParentId();
                endif;
                $array_cat[$categoryId] = $parentId;
                if ($subcategory->hasChildren()) {
                    $this->copySubCategory($categoryId, $newParentId, $array_cat);
                }
            }
        }
    }

    public function saveSubCategory($value, $key, &$extradata) {
        $parentdata = $this->getParentData($extradata['tree'][$key]);
        $model = $this->getCategory($key);
        $urlkey = $model->getUrlKey();
          // $this->output->writeln("\nCopying Sub Category: " . $model->getName() . "...");
        if ($extradata['data']['copy_sub_related']):
            $positions = $model->getProductsPosition();
            $model->setPostedProducts($positions);
            $model->setCategoryProducts(json_encode($positions));
            unset($model['products_position']);
        endif;
        $model->setUrlPath($parentdata['urlkey'] . '/' . $urlkey);
        if (isset($extradata['search']))
            $this->_SearchAndReplace($extradata['search'], $model);
        $model->setParentId($parentdata['id']);
        $model->setEntityId(null);
        $model->setPath($parentdata['path']);
        $model->setPathIds($arrayName = array('0' => ''));
        $model->setCreatedAt($this->_date->date()->format('Y-m-d H:i:s'));
        $model->setUpdatedAt(null);
        try {
            $model->save();
            $tree = $extradata['tree'];
            $extradata['tree'] = array_replace($tree, array_fill_keys(
                            array_keys($tree, $key), $model->getId()
            ));
        } catch (\Exception $e) {
            $result = ['error' => $e->getMessage(), 'errorcode' => $e->getCode()];
        }
    }

    public function _SearchAndReplace($_Search, &$model) {
        for ($i = 0; $i < sizeof($_Search); $i++) {
            $description = preg_replace('/\b' . $_Search[$i]['search'] . '\b/i', $_Search[$i]['replace'], $model->getDescription());
            $name = preg_replace('/\b' . $_Search[$i]['search'] . '\b/i', $_Search[$i]['replace'], $model->getName());
            $key = str_replace(' ', '-', strtolower($_Search[$i]['replace']));
            $urlpath = preg_replace('/\b' . $_Search[$i]['search'] . '\b/i', $key, $model->getUrlPath());
            $urlkey = preg_replace('/\b' . $_Search[$i]['search'] . '\b/i', $key, $model->getUrlKey());
            $model->setDescription($description);
            $model->setName($name);
            $model->setUrlPath($urlpath);
            $model->setUrlKey($urlkey);
        }
    }

    public function getParentData($parentid) {
        $data = array();
        $parentmodel = $this->getCategory($parentid);
        $data['id'] = $parentmodel->getEntityId();
        $data['path'] = $parentmodel->getPath();
        $data['urlkey'] = $parentmodel->getUrlKey();
        $data['level'] = $parentmodel->getLevel();
        return $data;
    }

    public function getCategory($categoryId) {
        $category = $this->categoryFactory->create()->load($categoryId);
        return $category;
    }

}
